@@ -3,19 +3,16 @@ import gzip
 import json
 import logging
 import time
-from asyncio import sleep
 from enum import Enum
-from random import random
-from socket import gaierror
+from random import random, randint
 from typing import Optional, List, Dict, Callable, Any
 
-import websockets as ws
-from websockets.exceptions import ConnectionClosedError
+import aiohttp
+from aiohttp import ClientWebSocketResponse
 
 from .client import AsyncClient
-from .enums import FuturesType
 from .exceptions import BinanceWebsocketUnableToConnect
-from .enums import ContractType
+from .enums import FuturesType
 from .threaded_stream import ThreadedApiManager
 
 KEEPALIVE_TIMEOUT = 5 * 60  # 5 minutes
@@ -25,6 +22,7 @@ class WSListenerState(Enum):
     INITIALISING = 'Initialising'
     STREAMING = 'Streaming'
     RECONNECTING = 'Reconnecting'
+    RECONNECTING_FAILED = "ReconnectingFailed"
     EXITING = 'Exiting'
 
 
@@ -37,17 +35,18 @@ class BinanceSocketType(str, Enum):
 
 
 class ReconnectingWebsocket:
+
     MAX_RECONNECTS = 5
     MAX_RECONNECT_SECONDS = 60
     MIN_RECONNECT_WAIT = 0.1
     TIMEOUT = 10
     NO_MESSAGE_RECONNECT_TIMEOUT = 60
-    MAX_QUEUE_SIZE = 100
 
     def __init__(
-        self, url: str, path: Optional[str] = None, prefix: str = 'ws/', is_binary: bool = False, exit_coro=None
+        self, loop, url: str, path: Optional[str] = None, prefix: str = 'ws/', is_binary: bool = False, exit_coro=None,
+        proxy=None
     ):
-        self._loop = asyncio.get_event_loop()
+        self._loop = loop or asyncio.get_event_loop()
         self._log = logging.getLogger(__name__)
         self._path = path
         self._url = url
@@ -57,10 +56,13 @@ class ReconnectingWebsocket:
         self._is_binary = is_binary
         self._conn = None
         self._socket = None
-        self.ws: Optional[ws.WebSocketClientProtocol] = None  # type: ignore
+        self._session = None
+        self.ws: Optional[ClientWebSocketResponse] = None
         self.ws_state = WSListenerState.INITIALISING
-        self._queue = asyncio.Queue()
-        self._handle_read_loop = None
+        self.reconnect_handle = None
+        self._proxy = proxy
+        self._topics = []
+        self._queue = asyncio.Queue(loop=self._loop)
 
     async def __aenter__(self):
         await self.connect()
@@ -71,35 +73,37 @@ class ReconnectingWebsocket:
             await self._exit_coro(self._path)
         self.ws_state = WSListenerState.EXITING
         if self.ws:
-            self.ws.fail_connection()
-        if self._conn and hasattr(self._conn, 'protocol'):
+            await self.ws.close()
+        if self._conn:
             await self._conn.__aexit__(exc_type, exc_val, exc_tb)
         self.ws = None
-        if not self._handle_read_loop:
-            self._log.error("CANCEL read_loop")
-            await self._kill_read_loop()
+        if self._session:
+            await self._session.close()
+            self._session = None
 
     async def connect(self):
         await self._before_connect()
         assert self._path
         ws_url = self._url + self._prefix + self._path
-        self._conn = ws.connect(ws_url, close_timeout=0.1)  # type: ignore
+        kwargs = {'url': ws_url}
+        if self._proxy:
+            kwargs['proxy'] = self._proxy
+        self._session = aiohttp.ClientSession(trust_env=True)
+        self._conn = self._session.ws_connect(**kwargs)
+
         try:
+            logging.debug("waiting connect")
             self.ws = await self._conn.__aenter__()
-        except:  # noqa
+        except Exception as e:  # noqa
+            logging.error(f"error: {e}")
+            self.ws_state = WSListenerState.RECONNECTING_FAILED
             await self._reconnect()
             return
+        logging.debug("connect successfully!")
         self.ws_state = WSListenerState.STREAMING
         self._reconnects = 0
         await self._after_connect()
-        # To manage the "cannot call recv while another coroutine is already waiting for the next message"
-        if not self._handle_read_loop:
-            self._handle_read_loop = self._loop.call_soon_threadsafe(asyncio.create_task, self._read_loop())
-
-    async def _kill_read_loop(self):
-        self.ws_state = WSListenerState.EXITING
-        while self._handle_read_loop:
-            await sleep(0.1)
+        self._loop.call_soon_threadsafe(asyncio.create_task, self._read_loop())
 
     async def _before_connect(self):
         pass
@@ -110,85 +114,59 @@ class ReconnectingWebsocket:
     def _handle_message(self, evt):
         if self._is_binary:
             try:
-                evt = gzip.decompress(evt)
+                evt = gzip.decompress(evt.data)
             except (ValueError, OSError):
                 return None
         try:
-            return json.loads(evt)
+            return json.loads(evt.data)
         except ValueError:
-            self._log.debug(f'error parsing evt json:{evt}')
+            self._log.debug(f'error parsing evt json:{evt.data}')
+            return None
+        except TypeError:
+            self._log.debug(f"invalid json object: {evt}")
             return None
 
     async def _read_loop(self):
-        try:
-            while True:
+        while True:
+            res = None
+            if not self.ws or self.ws_state != WSListenerState.STREAMING:
+                logging.debug(f"ws_state != STREAMING, wait for reconnect")
+                await self._wait_for_reconnect()
+                break
+            if self.ws_state == WSListenerState.EXITING:
+                logging.debug(f"ws_state == EXITING, break while")
+                break
+            if self.ws.closed:
                 try:
-                    if self.ws_state == WSListenerState.RECONNECTING:
-                        await self._run_reconnect()
-
-                    if not self.ws or self.ws_state != WSListenerState.STREAMING:
-                        await self._wait_for_reconnect()
-                        break
-                    elif self.ws_state == WSListenerState.EXITING:
-                        break
-                    elif self.ws.state == ws.protocol.State.CLOSING:  # type: ignore
-                        await asyncio.sleep(0.1)
-                        continue
-                    elif self.ws.state == ws.protocol.State.CLOSED:  # type: ignore
-                        await self._reconnect()
-                    elif self.ws_state == WSListenerState.STREAMING:
-                        res = await asyncio.wait_for(self.ws.recv(), timeout=self.TIMEOUT)
-                        res = self._handle_message(res)
-                        if res:
-                            if self._queue.qsize() < self.MAX_QUEUE_SIZE:
-                                await self._queue.put(res)
-                            else:
-                                self._log.debug(f"Queue overflow {self.MAX_QUEUE_SIZE}. Message not filled")
-                                await self._queue.put({
-                                    'e': 'error',
-                                    'm': 'Queue overflow. Message not filled'
-                                })
-                                raise BinanceWebsocketUnableToConnect
-                except asyncio.TimeoutError:
-                    self._log.debug(f"no message in {self.TIMEOUT} seconds")
-                    # _no_message_received_reconnect
-                except asyncio.CancelledError as e:
-                    self._log.debug(f"cancelled error {e}")
+                    await self._reconnect()
+                except BinanceWebsocketUnableToConnect:
+                    return {
+                        'e': 'error',
+                        'm': 'Max reconnect retries reached'
+                    }
+                else:
                     break
-                except asyncio.IncompleteReadError as e:
-                    self._log.debug(f"incomplete read error ({e})")
-                except ConnectionClosedError as e:
-                    self._log.debug(f"connection close error ({e})")
-                except gaierror as e:
-                    self._log.debug(f"DNS Error ({e})")
-                except BinanceWebsocketUnableToConnect as e:
-                    self._log.debug(f"BinanceWebsocketUnableToConnect ({e})")
+            try:
+                res = await asyncio.wait_for(self.ws.receive(), timeout=self.TIMEOUT)
+            except asyncio.TimeoutError:
+                logging.debug(f"no message in {self.TIMEOUT} seconds")
+            except asyncio.CancelledError as e:
+                logging.debug(f"cancelled error {e}")
+                break
+            except asyncio.IncompleteReadError as e:
+                logging.debug(f"incomplete read error {e}")
+            except Exception as e:
+                logging.debug(f"exception {e}")
+                break
+            else:
+                if self.ws_state in (WSListenerState.EXITING, WSListenerState.RECONNECTING):
+                    break
+                res = self._handle_message(res)
+                if self.ws_state in (WSListenerState.EXITING, WSListenerState.RECONNECTING):
                     break
-                except Exception as e:
-                    self._log.debug(f"Unknown exception ({e})")
-                    continue
-        finally:
-            self._handle_read_loop = None  # Signal the coro is stopped
-            self._reconnects = 0
 
-    async def _run_reconnect(self):
-        await self.before_reconnect()
-        if self._reconnects < self.MAX_RECONNECTS:
-            reconnect_wait = self._get_reconnect_wait(self._reconnects)
-            self._log.debug(
-                f"websocket reconnecting. {self.MAX_RECONNECTS - self._reconnects} reconnects left - "
-                f"waiting {reconnect_wait}"
-            )
-            await asyncio.sleep(reconnect_wait)
-            await self.connect()
-        else:
-            self._log.error(f'Max reconnections {self.MAX_RECONNECTS} reached:')
-            # Signal the error
-            await self._queue.put({
-                'e': 'error',
-                'm': 'Max reconnect retries reached'
-            })
-            raise BinanceWebsocketUnableToConnect
+            if res and self._queue.qsize() < 100:
+                await self._queue.put(res)
 
     async def recv(self):
         res = None
@@ -196,43 +174,127 @@ class ReconnectingWebsocket:
             try:
                 res = await asyncio.wait_for(self._queue.get(), timeout=self.TIMEOUT)
             except asyncio.TimeoutError:
-                self._log.debug(f"no message in {self.TIMEOUT} seconds")
+                logging.debug(f"no message in {self.TIMEOUT} seconds")
         return res
 
     async def _wait_for_reconnect(self):
-        while self.ws_state != WSListenerState.STREAMING and self.ws_state != WSListenerState.EXITING:
-            await sleep(0.1)
+        while self.ws_state == WSListenerState.RECONNECTING:
+            logging.debug("reconnecting waiting for connect")
+            await asyncio.sleep(1)
+        if not self.ws:
+            logging.debug("ignore message no ws")
+        else:
+            logging.debug(f"ignore message {self.ws_state}")
 
     def _get_reconnect_wait(self, attempts: int) -> int:
         expo = 2 ** attempts
         return round(random() * min(self.MAX_RECONNECT_SECONDS, expo - 1) + 1)
 
     async def before_reconnect(self):
-        if self.ws and self._conn:
+        if self.ws:
             await self._conn.__aexit__(None, None, None)
             self.ws = None
         self._reconnects += 1
 
     def _no_message_received_reconnect(self):
-        self._log.debug('No message received, reconnecting')
-        self.ws_state = WSListenerState.RECONNECTING
+        logging.debug('No message received, reconnecting')
+        asyncio.create_task(self._reconnect())
 
     async def _reconnect(self):
+        if self.ws_state == WSListenerState.RECONNECTING:
+            return
+        logging.debug("start to reconnect ...")
         self.ws_state = WSListenerState.RECONNECTING
+        await self.before_reconnect()
+        if self._reconnects < self.MAX_RECONNECTS:
+            reconnect_wait = self._get_reconnect_wait(self._reconnects)
+            logging.debug(
+                f"websocket reconnecting {self.MAX_RECONNECTS - self._reconnects} reconnects left - "
+                f"waiting {reconnect_wait}"
+            )
+            await asyncio.sleep(reconnect_wait)
+            logging.debug("start connect...")
+            await self.connect()
+            if self._topics:
+                await self.ws.send_json({
+                    "method": "SUBSCRIBE",
+                    "params": self._topics,
+                    "id": randint(1, 9999999999)
+                })
+        else:
+            logging.error(f'Max reconnections {self.MAX_RECONNECTS} reached:')
+            raise BinanceWebsocketUnableToConnect
+
+    async def _send(self, payload) -> bool:
+        if not self.ws or self.ws_state != WSListenerState.STREAMING:
+            await self._wait_for_reconnect()
+        if self.ws_state == WSListenerState.EXITING:
+            raise BinanceWebsocketUnableToConnect("ws_state is exiting")
+        try:
+            await asyncio.wait_for(self.ws.send_json(payload), timeout=self.TIMEOUT)
+            return True
+        except asyncio.TimeoutError:
+            logging.debug(f"no message in {self.TIMEOUT} seconds")
+        except asyncio.CancelledError as e:
+            logging.debug(f"cancelled error {e}")
+        except asyncio.IncompleteReadError as e:
+            logging.debug(f"incomplete read error {e}")
+        except Exception as e:
+            logging.debug(f"exception {e}")
+        return False
+
+    async def subscribe(self, topics: List[str]):
+        while True:
+            result = await self._send({
+                "method": "SUBSCRIBE",
+                "params": topics,
+                "id": randint(1, 9999999999)
+            })
+            if result:
+                break
+        self._topics.extend(topics)
+
+    async def unsubscribe(self, topics: List[str]):
+        while True:
+            result = await self._send({
+                "method": "UNSUBSCRIBE",
+                "params": topics,
+                "id": randint(1, 9999999999)
+            })
+            if result:
+                break
+        for _t in topics:
+            if _t in self._topics:
+                self._topics.remove(_t)
+
+    async def send_request(self, params: List[str]):
+        while True:
+            result = await self._send({
+                "method": "REQUEST",
+                "params": params,
+                "id": randint(1, 9999999999)
+            })
+            if result:
+                break
 
 
 class KeepAliveWebsocket(ReconnectingWebsocket):
-
-    def __init__(
-        self, client: AsyncClient, url, keepalive_type, prefix='ws/', is_binary=False, exit_coro=None,
-        user_timeout=None
-    ):
-        super().__init__(path=None, url=url, prefix=prefix, is_binary=is_binary, exit_coro=exit_coro)
+    def __init__(self, client: AsyncClient, loop, url, keepalive_type, prefix='ws/', is_binary=False, exit_coro=None, user_timeout=None, proxy=None):
+        super().__init__(loop=loop, path=None, url=url, prefix=prefix, is_binary=is_binary, exit_coro=exit_coro, proxy=proxy)
         self._keepalive_type = keepalive_type
         self._client = client
         self._user_timeout = user_timeout or KEEPALIVE_TIMEOUT
         self._timer = None
 
+    @property
+    def client(self):
+        return self._client
+
+    def get_listen_key(self):
+        if self._path:
+            return self._path
+        return None
+
     async def __aexit__(self, *args, **kwargs):
         if not self._path:
             return
@@ -251,7 +313,8 @@ class KeepAliveWebsocket(ReconnectingWebsocket):
     def _start_socket_timer(self):
         self._timer = self._loop.call_later(
             self._user_timeout,
-            lambda: asyncio.create_task(self._keepalive_socket())
+            asyncio.create_task,
+            self._keepalive_socket()
         )
 
     async def _get_listen_key(self):
@@ -269,32 +332,30 @@ class KeepAliveWebsocket(ReconnectingWebsocket):
         return listen_key
 
     async def _keepalive_socket(self):
-        try:
-            listen_key = await self._get_listen_key()
-            if listen_key != self._path:
-                self._log.debug("listen key changed: reconnect")
-                self._path = listen_key
-                await self._reconnect()
-            else:
-                self._log.debug("listen key same: keepalive")
-                if self._keepalive_type == 'user':
-                    await self._client.stream_keepalive(self._path)
-                elif self._keepalive_type == 'margin':  # cross-margin
-                    await self._client.margin_stream_keepalive(self._path)
-                elif self._keepalive_type == 'futures':
-                    await self._client.futures_stream_keepalive(self._path)
-                elif self._keepalive_type == 'coin_futures':
-                    await self._client.futures_coin_stream_keepalive(self._path)
-                else:  # isolated margin
-                    # Passing symbol for isolated margin
-                    await self._client.isolated_margin_stream_keepalive(self._keepalive_type, self._path)
-        except Exception:
-            pass  # Ignore
-        finally:
+        listen_key = await self._get_listen_key()
+
+        if listen_key != self._path:
+            logging.debug("listen key changed: reconnect")
+            self._path = listen_key
+            await self._reconnect()
+        else:
+            logging.debug("listen key same: keepalive")
+            if self._keepalive_type == 'user':
+                await self._client.stream_keepalive(self._path)
+            elif self._keepalive_type == 'margin':  # cross-margin
+                await self._client.margin_stream_keepalive(self._path)
+            elif self._keepalive_type == 'futures':
+                await self._client.futures_stream_keepalive(self._path)
+            elif self._keepalive_type == 'coin_futures':
+                await self._client.futures_coin_stream_keepalive(self._path)
+            else:  # isolated margin
+                # Passing symbol for isolated margin
+                await self._client.isolated_margin_stream_keepalive(self._keepalive_type, self._path)
             self._start_socket_timer()
 
 
 class BinanceSocketManager:
+
     STREAM_URL = 'wss://stream.binance.{}:9443/'
     STREAM_TESTNET_URL = 'wss://testnet.binance.vision/'
     FSTREAM_URL = 'wss://fstream.binance.{}/'
@@ -308,7 +369,7 @@ class BinanceSocketManager:
     WEBSOCKET_DEPTH_10 = '10'
     WEBSOCKET_DEPTH_20 = '20'
 
-    def __init__(self, client: AsyncClient, user_timeout=KEEPALIVE_TIMEOUT):
+    def __init__(self, client: AsyncClient, loop=None, user_timeout=KEEPALIVE_TIMEOUT, proxy=None):
         """Initialise the BinanceSocketManager
 
         :param client: Binance API client
@@ -322,9 +383,10 @@ class BinanceSocketManager:
         self.VSTREAM_TESTNET_URL = self.VSTREAM_TESTNET_URL.format(client.tld)
 
         self._conns = {}
-        self._loop = asyncio.get_event_loop()
+        self._loop = loop or asyncio.get_event_loop()
         self._client = client
         self._user_timeout = user_timeout
+        self._proxy = proxy
 
         self.testnet = self._client.testnet
 
@@ -343,11 +405,13 @@ class BinanceSocketManager:
         conn_id = f'{socket_type}_{path}'
         if conn_id not in self._conns:
             self._conns[conn_id] = ReconnectingWebsocket(
+                loop=self._loop,
                 path=path,
                 url=self._get_stream_url(stream_url),
                 prefix=prefix,
                 exit_coro=self._exit_socket,
-                is_binary=is_binary
+                is_binary=is_binary,
+                proxy=self._proxy
             )
 
         return self._conns[conn_id]
@@ -359,12 +423,14 @@ class BinanceSocketManager:
         if conn_id not in self._conns:
             self._conns[conn_id] = KeepAliveWebsocket(
                 client=self._client,
+                loop=self._loop,
                 url=self._get_stream_url(stream_url),
                 keepalive_type=path,
                 prefix=prefix,
                 exit_coro=self._exit_socket,
                 is_binary=is_binary,
-                user_timeout=self._user_timeout
+                user_timeout=self._user_timeout,
+                proxy=self._proxy
             )
 
         return self._conns[conn_id]
@@ -508,56 +574,6 @@ class BinanceSocketManager:
         path = f'{symbol.lower()}@kline_{interval}'
         return self._get_socket(path)
 
-    def kline_futures_socket(self, symbol: str, interval=AsyncClient.KLINE_INTERVAL_1MINUTE,
-                             futures_type: FuturesType = FuturesType.USD_M,
-                             contract_type: ContractType = ContractType.PERPETUAL):
-        """Start a websocket for symbol kline data for the perpeual futures stream
-
-        https://binance-docs.github.io/apidocs/futures/en/#continuous-contract-kline-candlestick-streams
-
-        :param symbol: required
-        :type symbol: str
-        :param interval: Kline interval, default KLINE_INTERVAL_1MINUTE
-        :type interval: str
-        :param futures_type: use USD-M or COIN-M futures default USD-M
-        :param contract_type: use PERPETUAL or CURRENT_QUARTER or NEXT_QUARTER default PERPETUAL
-
-        :returns: connection key string if successful, False otherwise
-
-        Message Format
-
-        .. code-block:: python
-
-                {
-                "e":"continuous_kline",   // Event type
-                "E":1607443058651,        // Event time
-                "ps":"BTCUSDT",           // Pair
-                "ct":"PERPETUAL"          // Contract type
-                "k":{
-                    "t":1607443020000,      // Kline start time
-                    "T":1607443079999,      // Kline close time
-                    "i":"1m",               // Interval
-                    "f":116467658886,       // First trade ID
-                    "L":116468012423,       // Last trade ID
-                    "o":"18787.00",         // Open price
-                    "c":"18804.04",         // Close price
-                    "h":"18804.04",         // High price
-                    "l":"18786.54",         // Low price
-                    "v":"197.664",          // volume
-                    "n": 543,               // Number of trades
-                    "x":false,              // Is this kline closed?
-                    "q":"3715253.19494",    // Quote asset volume
-                    "V":"184.769",          // Taker buy volume
-                    "Q":"3472925.84746",    //Taker buy quote asset volume
-                    "B":"0"                 // Ignore
-                }
-            }
-            <pair>_<contractType>@continuousKline_<interval>
-        """
-
-        path = f'{symbol.lower()}_{contract_type.value}@continuousKline_{interval}'
-        return self._get_futures_socket(path, prefix='ws/', futures_type=futures_type)
-
     def miniticker_socket(self, update_time: int = 1000):
         """Start a miniticker websocket for all trades
 
@@ -760,9 +776,6 @@ class BinanceSocketManager:
 
         https://github.com/binance-exchange/binance-official-api-docs/blob/master/web-socket-streams.md#all-market-tickers-stream
 
-        :param coro: callback function to handle messages
-        :type coro: function
-
         :returns: connection key string if successful, False otherwise
 
         Message Format
@@ -816,18 +829,9 @@ class BinanceSocketManager:
         stream_name = '@indexPrice@1s' if fast else '@indexPrice'
         return self._get_futures_socket(symbol.lower() + stream_name, futures_type=FuturesType.COIN_M)
 
-    def futures_depth_socket(self, symbol: str, depth: str = '10', futures_type=FuturesType.USD_M):
-        """Subscribe to a futures depth data stream
-
-        https://binance-docs.github.io/apidocs/futures/en/#partial-book-depth-streams
-
-        :param symbol: required
-        :type symbol: str
-        :param depth: optional Number of depth entries to return, default 10.
-        :type depth: str
-        :param futures_type: use USD-M or COIN-M futures default USD-M
-        """
-        return self._get_futures_socket(symbol.lower() + '@depth' + str(depth), futures_type=futures_type)
+    def multi_index_price_socket(self, symbols: List[str], fast: bool = True):
+        stream_names = [symbol.lower() + ('@indexPrice@1s' if fast else '@indexPrice') for symbol in symbols]
+        return self._get_futures_socket('/'.join(stream_names), futures_type=FuturesType.COIN_M)
 
     def symbol_mark_price_socket(self, symbol: str, fast: bool = True, futures_type: FuturesType = FuturesType.USD_M):
         """Start a websocket for a symbol's futures mark price
@@ -934,6 +938,17 @@ class BinanceSocketManager:
 
         return self._get_futures_socket('!bookTicker', futures_type=futures_type)
 
+    def future_depth_socket(self, symbol: str, depth: str = '10', futures_type: FuturesType = FuturesType.USD_M):
+        """Subscribe to a depth data stream
+
+        :param symbol: required
+        :type symbol: str
+        :param depth: optional Number of depth entries to return, default 10.
+        :type depth: str
+        :param futures_type:
+        """
+        return self._get_futures_socket(symbol.lower() + '@depth' + str(depth), futures_type=futures_type)
+
     def symbol_book_ticker_socket(self, symbol: str):
         """Start a websocket for the best bid or ask's price or quantity for a specified symbol.
 
@@ -1045,8 +1060,8 @@ class BinanceSocketManager:
     def user_socket(self):
         """Start a websocket for user data
 
-        https://github.com/binance-exchange/binance-official-api-docs/blob/master/user-data-stream.md
-        https://binance-docs.github.io/apidocs/spot/en/#listen-key-spot
+            https://github.com/binance-exchange/binance-official-api-docs/blob/master/user-data-stream.md
+            https://binance-docs.github.io/apidocs/spot/en/#listen-key-spot
 
         :returns: connection key string if successful, False otherwise
 
@@ -1054,18 +1069,6 @@ class BinanceSocketManager:
         """
         return self._get_account_socket('user')
 
-    def futures_user_socket(self):
-        """Start a websocket for coin futures user data
-
-        https://binance-docs.github.io/apidocs/futures/en/#user-data-streams
-
-        :returns: connection key string if successful, False otherwise
-
-        Message Format - see Binanace API docs for all types
-        """
-
-        return self._get_account_socket('futures', stream_url=self.FSTREAM_URL)
-
     def margin_socket(self):
         """Start a websocket for cross-margin data
 
@@ -1097,10 +1100,7 @@ class BinanceSocketManager:
 
         Message Format - see Binance API docs for all types
         """
-        stream_url = self.DSTREAM_URL
-        if self.testnet:
-            stream_url = self.DSTREAM_TESTNET_URL
-        return self._get_account_socket('coin_futures', stream_url=stream_url)
+        return self._get_account_socket('coin_futures', stream_url=self.DSTREAM_URL)
 
     def isolated_margin_socket(self, symbol: str):
         """Start a websocket for isolated margin data
@@ -1171,14 +1171,14 @@ class BinanceSocketManager:
         if conn_key not in self._conns:
             return
 
-        del (self._conns[conn_key])
+        del(self._conns[conn_key])
 
 
 class ThreadedWebsocketManager(ThreadedApiManager):
 
     def __init__(
         self, api_key: Optional[str] = None, api_secret: Optional[str] = None,
-        requests_params: Optional[Dict[str, str]] = None, tld: str = 'com',
+        requests_params: Dict[str, str] = None, tld: str = 'com',
         testnet: bool = False
     ):
         super().__init__(api_key, api_secret, requests_params, tld, testnet)
@@ -1186,7 +1186,7 @@ class ThreadedWebsocketManager(ThreadedApiManager):
 
     async def _before_socket_listener_start(self):
         assert self._client
-        self._bsm = BinanceSocketManager(client=self._client)
+        self._bsm = BinanceSocketManager(client=self._client, loop=self._loop)
 
     def _start_async_socket(
         self, callback: Callable, socket_name: str, params: Dict[str, Any], path: Optional[str] = None
@@ -1194,10 +1194,10 @@ class ThreadedWebsocketManager(ThreadedApiManager):
         while not self._bsm:
             time.sleep(0.1)
         socket = getattr(self._bsm, socket_name)(**params)
-        socket_path: str = path or socket._path  # noqa
-        self._socket_running[socket_path] = True
-        self._loop.call_soon_threadsafe(asyncio.create_task, self.start_listener(socket, socket_path, callback))
-        return socket_path
+        path = path or socket._path  # noqa
+        self._socket_running[path] = True
+        self._loop.call_soon_threadsafe(asyncio.create_task, self.start_listener(socket, socket._path, callback))
+        return path
 
     def start_depth_socket(
         self, callback: Callable, symbol: str, depth: Optional[str] = None, interval: Optional[int] = None
@@ -1222,21 +1222,6 @@ class ThreadedWebsocketManager(ThreadedApiManager):
             }
         )
 
-    def start_kline_futures_socket(self, callback: Callable, symbol: str,
-                                   interval=AsyncClient.KLINE_INTERVAL_1MINUTE,
-                                   futures_type: FuturesType = FuturesType.USD_M,
-                                   contract_type: ContractType = ContractType.PERPETUAL) -> str:
-        return self._start_async_socket(
-            callback=callback,
-            socket_name='kline_futures_socket',
-            params={
-                'symbol': symbol,
-                'interval': interval,
-                'futures_type': futures_type,
-                'contract_type': contract_type
-            }
-        )
-
     def start_miniticker_socket(self, callback: Callable, update_time: int = 1000) -> str:
         return self._start_async_socket(
             callback=callback,
@@ -1422,13 +1407,6 @@ class ThreadedWebsocketManager(ThreadedApiManager):
             params={}
         )
 
-    def start_futures_user_socket(self, callback: Callable) -> str:
-        return self._start_async_socket(
-            callback=callback,
-            socket_name='futures_user_socket',
-            params={}
-        )
-
     def start_margin_socket(self, callback: Callable) -> str:
         return self._start_async_socket(
             callback=callback,
@@ -1498,14 +1476,3 @@ class ThreadedWebsocketManager(ThreadedApiManager):
                 'depth': depth
             }
         )
-
-    def start_futures_depth_socket(self, callback: Callable, symbol: str, depth: str = '10', futures_type=FuturesType.USD_M) -> str:
-        return self._start_async_socket(
-            callback=callback,
-            socket_name='futures_depth_socket',
-            params={
-                'symbol': symbol,
-                'depth': depth,
-                'futures_type': futures_type
-            }
-        )
